/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2005 by Walther Neuper
 * created Jun 10, 2005 10:44:45 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.bridge;

import isac.kestore.KEStoreServices;
import isac.util.CalcChangedEvent;
import isac.util.Formalization;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.CalcHead;
import isac.util.formulae.KEStoreID;
import isac.util.formulae.Match;
import isac.util.formulae.Position;
import isac.util.formulae.ProblemID;
import isac.util.interfaces.ICalcIterator;
import isac.util.interfaces.IToCalc;
import isac.util.interfaces.IToUser;
import isac.util.parser.FormalizationDigest;
import isac.util.tactics.SimpleTactic;
import isac.util.tactics.StringListTactic;
import isac.util.tactics.Tactic;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.Vector;

import junit.framework.TestCase;

/**
 * tests kernel in the specify-phase
 * 
 * @author Walther Neuper
 * @version 1.0
 */
public class TestModel extends TestCase {

    Formalization fmz_;

    MathEngine me_;

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_7a.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        fmz_ = fd.parseFormalization((String) xmlstr);

        MathEngine.init("localhost");
        me_ = MathEngine.getMathEngine();
    }

    /**
     * interactively complete the Model by using the <NEXT>button;; direct calls
     * of the math-engine
     */
    public void testAddItems() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestModel#testAddItems");

        //start the calculation
        IToCalc ct = me_.startCalculation(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChangedEvent cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //fill the Models Items with Descriptions for userguidance:
        //tac = Model_Problem ...
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        ch = (CalcHead) ci.getElement();
        assertEquals(ch.getModel().getGiven().toString(),
                "[\"incomplete fixedValues []\"]");
        System.out.println(".end step_b-----");
        //end step_b ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //fill the Models Items stepwise with the <Next>-button
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        ch = (CalcHead) ci.getElement();
        assertEquals(ch.getModel().getGiven().toString(),
                "[\"correct fixedValues [r = Arbfix]\"]");
        //and so on...
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        //lists are filled elementwise:
        ch = (CalcHead) ci.getElement();
        assertEquals(
                ch.getModel().getFind().getItems().elementAt(1).toString(),
                "incomplete valuesFor [a]");
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        ch = (CalcHead) ci.getElement();
        assertEquals(
                ch.getModel().getFind().getItems().elementAt(1).toString(),
                "correct valuesFor [a, b]");
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        ch = (CalcHead) ci.getElement();
        System.out.println(ch.getModel().toString());
        //now the Model is complete; for completing the CalcHead by completing
        // the Specification see testEditItems
        System.out.println(".end step_c-----");
        //end step_c ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        System.out.println("---END isac.bridge.TestModel#testAddItems");
    }

    /**
     * {\bf\UC{Having \isac{} Complete the
     * CalcHead}\label{SPECIFY:COMPLETE:all}\\} The user requests a CalcHead,
     * containing some input, to be completed automatically. WN0506 What kind of
     * messed input should be overwritten/corredted ???
     * 
     * @see isac.functest.specify.TestModel#testCompleteCalcHead() for how this
     *      looks like on the CalcHeadPanel
     */
    public void testCompleteCalcHead() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestModel#testCompleteCalcHead");

        //start the calculation
        IToCalc ct = me_.startCalculation(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChangedEvent cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the CalcHead with ONE SPECIAL BUTTON ...
        ct.completeCalcHead();
        ch = (CalcHead) ci.getElement();
        //the model is complete
        assertEquals(ch.getModel().getGiven().getItems().elementAt(0)
                .toString(), "correct fixedValues [r = Arbfix]");
        assertEquals(
                ch.getModel().getFind().getItems().elementAt(0).toString(),
                "correct maximum A");
        assertEquals(
                ch.getModel().getFind().getItems().elementAt(1).toString(),
                "correct valuesFor [a, b]");
        assertEquals(ch.getModel().getRelate().getItems().elementAt(0)
                .toString(), "correct relations [A = a * b, "
                + "(a / 2) ^^^ 2 + (b / 2) ^^^ 2 = r ^^^ 2]");
        //the Specification is complete
        assertEquals(ch.getSpecification().toSMLString(), "\"DiffApp.thy\","
                + "[\"maximum_of\",\"function\"],"
                + "[\"DiffApp\",\"max_by_calculus\"]");
        System.out.println(".end step_b-----");
        //end step_b
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        System.out.println("---END isac.bridge.TestModel#testCompleteCalcHead");
    }

    /**
     * complete the Model by hitting ONE SPECIFIC BUTTON. WN0506 What kind of
     * messed input should be overwritten/corredted ???
     * 
     * @see isac.functest.specify.TestModel#testCompleteModel() for how this
     *      looks like on the CalcHeadPanel
     */
    public void testCompleteModel() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestModel#testCompleteModel");

        //start the calculation
        IToCalc ct = me_.startCalculation(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChangedEvent cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Model with ONE SPECIAL BUTTON ...
        ct.autoCalculate(IToCalc.SCOPE_MODEL, 0);
        ch = (CalcHead) ci.getElement();
        assertEquals(ch.getModel().getGiven().getItems().elementAt(0)
                .toString(), "correct fixedValues [r = Arbfix]");
        assertEquals(
                ch.getModel().getFind().getItems().elementAt(0).toString(),
                "correct maximum A");
        assertEquals(
                ch.getModel().getFind().getItems().elementAt(1).toString(),
                "correct valuesFor [a, b]");
        assertEquals(ch.getModel().getRelate().getItems().elementAt(0)
                .toString(), "correct relations [A = a * b, "
                + "(a / 2) ^^^ 2 + (b / 2) ^^^ 2 = r ^^^ 2]");
        System.out.println(".end step_b-----");
        //end step_b
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        System.out.println("---END isac.bridge.TestModel#testCompleteModel");
    }

    /**
     * complete the Model by manual input of the Items; any number of inputs;;
     * direct calls of the math-engine
     * 
     * @see isac.functest.specify.TestModel#testEditItems() for how this looks
     *      like on the CalcHeadPanel
     */
    public void testEditItems() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestModel#testEditItems");

        //start the calculation
        IToCalc ct = me_.startCalculation(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChangedEvent cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //TOOODO.WN050627 goon here

        System.out.println("---END isac.bridge.TestModel#testEditItems");
    }

    /**
     * {\bf\UC{Having \isac{} Complete One Field of the
     * Model}\label{SPECIFY:COMPLETE:modelone}\\}
     * 
     * @see isac.functest.specify.TestModel#testCompleteOneFieldOfModel() for
     *      how this looks like on the CalcHeadPanel
     * 
     * related SML is not implemented; would also require the bridge to send the
     * Items' status from Java to SML
     */
    public void testCompleteOneFieldOfModel() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestModel#testCompleteOneFieldOfModel");
        System.out.println("TODO");
        assertEquals("TODO", "TODO");
        System.out
                .println("---END isac.bridge.TestModel#testCompleteOneFieldOfModel");
    }

    /**
     * hit the 'Problems'-button on the front-end while the activeFormula is on
     * the Worksheet;; direct calls of the math-engine (and NOT on a
     * CalcHeadPanel)
     * 
     * This testcase requires to run a different example as defined in the
     * setup.
     */
    public void testKBfromWorksheet() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestBridgeSpecify#testKBfromWorksheet");
        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1b.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();
        IToCalc ct = me.startCalculation(fmz);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChangedEvent cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the example and set the activeFormula somewhere on the
        // Worksheet
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 0);
        //set the activeFormula within the root-problem
        
        
        
        System.out
                .println("---END isac.bridge.TestBridgeSpecify#testKBfromWorksheet");
    }

}